;;
;; Copyright (C) 2013-2018 Aleix Conchillo Flaque <aconchillo@gmail.com>
;;
;; This file is part of guile-json.
;;
;; guile-json is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; guile-json is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with guile-json. If not, see https://www.gnu.org/licenses/.
;;

(package (json (4 3 2))
  (depends (srfi-1))
  (synopsis "JSON parser/writer for Guile")
  (description
   "guile-json supports the parsing and writing of JSON from Guile."
   "It aims to be fully compliant with the json.org specification, and"
   "is released under the GPLv3.")
  (homepage "https://github.com/aconchillo/guile-json")
  (libraries "json.scm" "json")
  (documentation "README" "COPYING"))
